-- something that represents a level camera, polygon sotrage and renderer

local Level={
	fov=math.rad(60),
	near=0.01,
	far=1000,
	aspectRatio=love.graphics.getWidth()/love.graphics.getHeight(),
	
	-- default glsl shader for rendering
	shader=Shader.getDefault(),
	
	-- view matrix for camera
	view=Matrix.identity(),
	
	-- projection matrix for the screen should be ortographic
	projection=Matrix.identity(),
	
	-- camera position and zoom
	x=0,
	y=0,
	zoom=0,
	
	-- polygons to draw
	polygons={},
	
	help=[[
Demo made by Rigachupe.
Both polygons are a 4x4 tile made from 2 triangles with a pivot point [0,0] in the middle. Each uses it's own texture.
a,s,d,w - move camera
z,x - zoom camera
f - toggle filter nearest/linear
i,j,k,l - move polygon
o,p - rotate polygon
c - toggle wireframe on/off
escape - exit
	]]
}

function Level.test()
	-- make some rock quad with 15 degree rotation
	local polygon1=Polygon.test("data/rock.png",true)
	Polygon.setRotation(polygon1,math.rad(15))
	Polygon.updateMatrix(polygon1)
	
	-- make some sand quad with position away from rock quad
	local polygon2=Polygon.test("data/sand.png",false)
	Polygon.setPosition(polygon2,20,-5)
	Polygon.updateMatrix(polygon2)
	
	-- add polygons to level
	Level.addPolygon(polygon1)
	Level.addPolygon(polygon2)
	
	-- set best camera view for level
	Level.setCamera(0,0,-30)
end

-- add the polygon to level polygons
function Level.addPolygon(polygon)
	table.insert(Level.polygons,polygon)
end

-- set the camera to position x,y and the z is like a zoom factor
function Level.setCamera(x,y,zoom)
	Level.x=x
	Level.y=y
	Level.zoom=zoom
	Level.updateCamera()
end

-- update camera only when the camera position changes
function Level.updateCamera()
	Matrix.updateView(Level.view,Level.x,Level.y,Level.zoom)
	Matrix.updateProjection(Level.projection,Level.fov,Level.near,Level.far,Level.aspectRatio)
	
	Shader.send(Level.shader,"viewMatrix",Level.view)
	Shader.send(Level.shader,"projectionMatrix",Level.projection)
end

-- draw the level with polygons
function Level.draw()
	-- set the shader rendering
	Shader.apply(Level.shader)
	
	if Window.wireframe then
		love.graphics.setWireframe(true)
	end
	
	-- render polygons
	love.graphics.setColor(1,1,1,1)
	for index,polygon in ipairs(Level.polygons) do
		Polygon.draw(Level.shader,polygon)
	end
	
	love.graphics.setWireframe(false)
	
	-- turn of shader
	Shader.apply()
	
	-- and i can render normally
	love.graphics.setColor(1,1,1,1)
	love.graphics.print("Camera at:"..Level.x..","..Level.y..","..Level.zoom,10,10)
	love.graphics.print(Level.help,10,30)
end

-- the keys that handle the demo output
function Level.keypressed(key)
	if key=="a" then
		Level.x=Level.x-1
		Level.updateCamera()
	elseif key=="d" then
		Level.x=Level.x+1
		Level.updateCamera()
	elseif key=="w" then
		Level.y=Level.y-1
		Level.updateCamera()
	elseif key=="s" then
		Level.y=Level.y+1
		Level.updateCamera()
	elseif key=="z" then
		if Level.zoom>-200 then
			Level.zoom=Level.zoom-1
		end
		Level.updateCamera()
	elseif key=="x" then
		if Level.zoom<-20 then
			Level.zoom=Level.zoom+1
		end
		Level.updateCamera()
	elseif key=="o" then
		local p=Level.polygons[1]
		p.rotation[3]=p.rotation[3]+0.05
		Polygon.updateMatrix(p)
	elseif key=="p" then
		local p=Level.polygons[1]
		p.rotation[3]=p.rotation[3]-0.05
		Polygon.updateMatrix(p)
	elseif key=="j" then
		local p=Level.polygons[1]
		p.position[1]=p.position[1]-0.5
		Polygon.updateMatrix(p)
	elseif key=="l" then
		local p=Level.polygons[1]
		p.position[1]=p.position[1]+0.5
		Polygon.updateMatrix(p)
	elseif key=="i" then
		local p=Level.polygons[1]
		p.position[2]=p.position[2]+0.5
		Polygon.updateMatrix(p)
	elseif key=="k" then
		local p=Level.polygons[1]
		p.position[2]=p.position[2]-0.5
		Polygon.updateMatrix(p)
	elseif key=="f" then
		Window.toggleDefaultFilter()
	elseif key=="c" then
		Window.toggleWireframe()
	elseif key=="escape" then
		love.event.quit(0)
	end
end

return Level